import sys
import os
import math
import fractions
import urllib2

APPNAME="CameraLeash"

#the version.py file is generated by 'make config'. If it fails to import,
# show a message indicating that.
try:
	from version  import *
except Exception as ex:
	print "Could not find version.py. Perhaps you need to run 'make config'?"
	sys.exit()
	

def to_fraction(float):
	"""
	convert a float into a useful fraction, e.g
	to_fraction(0.1) -> '1/10'
	"""
	if float >= 1: return str(float)
	return str(fractions.Fraction(float).limit_denominator(10000))
	
	
def event_text(event_type):
		event_text = {
			gp.GP_EVENT_CAPTURE_COMPLETE: "Capture Complete",
			gp.GP_EVENT_FILE_ADDED: "File Added",
			gp.GP_EVENT_FOLDER_ADDED: "Folder Added",
			gp.GP_EVENT_TIMEOUT: "Timeout"
		}
		if event_type in event_text: return event_text[event_type]
		else: return "Unknown Event"
	
def hunam_size(size,kb = False):
	"""
	return a hunam-readable file size from a number of bytes
	"""
	if size == 0:
		return "0 B"
	size_name = ["B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB"]
	if kb:
		discarded = size_name.pop(0)
	i = int(math.floor(math.log(size, 1024)))
	p = math.pow(1024, i)
	s = round(size / p, 2)
	return "%s %s" % (s, size_name[i])

def backtick(command):
	"""
	Act like bash's backtick function - run a command and return its output
	"""
	p = os.popen("%s" % command,"r")
	output = ""
	while 1:
		line = p.readline()
		output = output + line
		if not line: break
	return output.strip()

def http_request(url,data = None):
	"""
	A Simple HTTP GET/POST
	- If you provide data, you're doing a post, otherwise a GET.
	"""
	if data:
		method = "POST"
	else:
		method = "GET"
	print("HTTP %s: '%s'" % (method,url))
	request = urllib2.Request(url,data)
	request.add_header('User-Agent', "%s v%01.02f" % (APPNAME,VERSION))
	result = urllib2.urlopen(request).readlines()
	ret = ""
	for line in result:
		if ret:
			ret += "\n"
		ret += line.strip()
	return ret

def read_file(filename):
	"""
	Returns a list of lines
	"""
	with open (filename, "r") as myfile:
		data=myfile.readlines()
	return data

def read_file_str(filename):
	"""
	Returns one big string
	"""
	return "\n".join(read_file(filename))


def is_pandora():
	"""
	Return a bool indicating whether we're running on an OpenPandora
	"""
	return os.path.exists("/etc/pandora") and os.path.isdir('/etc/pandora')

def usb_host_enabled():
	"""
	Return a bool indicating whether USB Host mode is enabled.
	If we're not running on a pandora, we assume it is.
	"""
	if not is_pandora(): return True
	
	return not ( backtick("lsmod | grep ehci_hcd") == "" )
